﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace As.UCCNCplugins.Forms
{
    internal partial class Main : Form
    {
        #region Instance
        public Main(UCCNCplugin plugin)
        {
            InitializeComponent();
            InitialiseLog();
            toolStripStatusLabel.Text = "initialiseren.";

            InitialiseApplication(plugin);
            InitialiseForm();

            StatusClear();
        }

        ~Main() { Finalise(); }

        void Finalise()
        {
            FinaliseApplication();
            FinaliseLog();
        }
        #endregion Instance

        #region Application
        void InitialiseApplication(UCCNCplugin plugin)
        {
            if (plugin == null)
            {
                UCCNCplugin.LogWriteLine("Form: action = \"initialise application\"; problem = \"No plugin instance\"");
                return;
            }
            UCCNCplugin.LogWriteLine("Form: action = \"initialise application\"");

            this.plugin = plugin;
            plugin.OnClockTick += OnClockTick;
        }

        void FinaliseApplication()
        {
            if (plugin == null) return;

            plugin.OnClockTick -= OnClockTick;
            plugin = null;
        }

        UCCNCplugin plugin = null;

        void OnClockTick()
        {
            string message = null;
            while (UCCNCplugin.LogTryRead(ref message)) LogWriteLn(message);

            toolStripStatusLabelPosition.Text = string.Format(
                (string)toolStripStatusLabelPosition.Tag,
                plugin?.X ?? "-",
                plugin?.Y ?? "-",
                plugin?.Z ?? "-"
            );

            if (plugin?.IsMoving ?? false)
            {
                textBoxMDI.Enabled = false;
                toolStripStatusLabelMoving.ForeColor = Color.Black;
                toolStripStatusLabelMoving.BackColor = Color.Red;
            }
            else
            {
                textBoxMDI.Enabled = true;
                toolStripStatusLabelMoving.ForeColor = SystemColors.ControlText;
                toolStripStatusLabelMoving.BackColor = SystemColors.Control;
            }
        }
        #endregion Application

        #region Log and status.
        void InitialiseLog()
        {
            #if DEBUG
            var show_log_controls = true;
            #else
            var show_log_controls = false;
            #endif

            log_useTimestamps = false;                              // true: in scherm ook datum/tijd vermelden.
            log_writeOnTop = true;                                  // true: in scherm schrijf nieuwe regels bovenaan.
            if (show_log_controls)
            {
                log_showExceptions = true;                          // false: toon exceptions, alleen het message deel.
                checkBoxLogExceptions.Visible = true;              // true: gebruiker kan het tonen van excepties aanpassen.
                splitContainerMainLog.Panel2Collapsed = true;   // true: toon de log per default
                checkBoxExceptions_Click(this, null);
            }
            else
            {
                log_showExceptions = false;                         // false: toon exceptions, alleen het message deel.
                checkBoxLogExceptions.Visible = false;             // false: gebruiker kan het tonen van excepties niet aanpassen.
                splitContainerMainLog.Panel2Collapsed = false;  // false: toon de log per default niet.
            }
            toolStripButtonShowLog_Click(null, null);
        }

        void FinaliseLog()
        { }

        /// <summary>
        /// log_showExceptions - true: toon exceptions met volledige stack trace in scherm
        /// </summary>
        bool log_showExceptions;

        /// <summary>
        /// log_useTimestamps - true: in scherm ook datum/tijd vermelden.
        /// </summary>
        bool log_useTimestamps;

        /// <summary>
        /// // true: in scherm schrijf nieuwe regels bovenaan.
        /// </summary>
        bool log_writeOnTop;

        delegate void PLogClear();

        /// <summary>
        /// LogClear - Clear the log text box.
        /// </summary>
        void LogClear()
        {
            if (InvokeRequired) Invoke(new PLogClear(LogClear));
            else textBoxLog.Text = string.Empty;
        }

        delegate void PLogWriteLn();

        void LogWriteLn()
        {
            if (InvokeRequired) Invoke(new PLogWriteLn(LogWriteLn));
            else
            {
                if (textBoxLog.Text.Length > 0)
                {
                    if (log_writeOnTop)
                    {
                        textBoxLog.Text = Environment.NewLine + textBoxLog.Text;
                        LogTrunk();
                    }
                    else textBoxLog.Text += Environment.NewLine;
                }
            }
        }

        delegate void PLogWriteLn_string(string msg);

        void LogWriteLn(string msg)
        {
            if (textBoxLog.InvokeRequired) Invoke(new PLogWriteLn_string(LogWriteLn), new object[] { msg });
            else
            {
                string line = string.Empty;
                if (log_useTimestamps) line += string.Format("{0}: ", DateTime.Now.ToString("HH:mm:ss.ffff"));
                line += msg;

                if (log_writeOnTop)
                {
                    textBoxLog.Text = (textBoxLog.Text.Length > 0)
                        ? line + Environment.NewLine + textBoxLog.Text
                        : line;
                    LogTrunk();
                }
                else if ((textBoxLog.Text.Length > 0)) textBoxLog.Text += Environment.NewLine + line;
                else textBoxLog.Text = line;

                Refresh();
            }
        }

        const int LOG_MAX_LENGTH = 4096;

        void LogTrunk()
        {
            if (LOG_MAX_LENGTH < textBoxLog.Text.Length)
            {
                var i = textBoxLog.Text.IndexOf('\n', LOG_MAX_LENGTH);
                if (-1 < i) textBoxLog.Text = textBoxLog.Text.Substring(0, i);
            }
        }

        private void toolStripButtonShowLog_Click(object sender, EventArgs e)
        {
            if (splitContainerMainLog.Panel2Collapsed)
            {
                toolStripButtonShowLog.Text = "Hide Log";
                splitContainerMainLog.Panel2Collapsed = false;
            }
            else
            {
                toolStripButtonShowLog.Text = "Show Log";
                splitContainerMainLog.Panel2Collapsed = true;
            }
        }

        void buttonLogClear_Click(object sender, EventArgs e)
        {
            LogClear();
        }

        void checkBoxExceptions_Click(object sender, EventArgs e)
        {
            log_showExceptions = !log_showExceptions;
            checkBoxLogExceptions.Checked = log_showExceptions;
            LogWriteLn($"Showing exceptions is set to {log_showExceptions}");
        }

        void StatusClear() { StatusUpdate(true); }

        void StatusUpdate(string message = "Problem")
        {
            StatusUpdate(false, message);
        }

        void StatusUpdate(bool result, string failing = "Problem", string ok = "OK")
        {
            toolStripStatusLabel.Text = (result) ? ok : failing;
        }
        #endregion Log and status.

        #region Form
        void InitialiseForm()
        {
            toolStripStatusLabelMoving.Alignment = ToolStripItemAlignment.Right;
            Text = $"{UCCNCplugin.Product}, {UCCNCplugin.Version}";
        }

        private void Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            Finalise();
        }

        private void buttonGetStatistics_Click(object sender, EventArgs e)
        {
            plugin?.GetStatistics();
        }

        private void buttonPosition_Click(object sender, EventArgs e)
        {
            plugin?.GetPosition();
        }
        #endregion Form

        private void textBoxMDI_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.KeyData)
            {
            case Keys.Enter:
                if (plugin?.WriteMDI(textBoxMDI.Text) ?? false) textBoxMDI.Text = "";
                break;
            }
        }
    }
}
